"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ASTPrinter = void 0;
const core_1 = require("@vulcan-sql/core");
const nunjucks = require("nunjucks");
const fs = require("fs");
const path = require("path");
class ASTPrinter extends core_1.CompileTimeExtension {
    constructor() {
        var _a;
        super(...arguments);
        this.treeIndex = 0;
        this.debugFolder = ((_a = this.getConfig()) === null || _a === void 0 ? void 0 : _a['debug-folder']) ||
            path.resolve(process.cwd(), '.vulcan-debug');
    }
    onVisit(node) {
        if (!(node instanceof nunjucks.nodes.Root))
            return;
        const mermaidString = this.compilerToMermaid(node);
        fs.mkdirSync(this.debugFolder, { recursive: true });
        fs.writeFileSync(path.resolve(this.debugFolder, `ast-${this.treeIndex++}.md`), mermaidString, 'utf-8');
    }
    compilerToMermaid(ast) {
        let output = '';
        const append = (str, indent = 0) => {
            output += ' '.repeat(indent) + str + '\n';
        };
        append('graph LR');
        let nodeIndex = 0;
        const bindId = (node) => {
            node.id = nodeIndex;
            return nodeIndex++;
        };
        bindId(ast);
        const walkAst = (root) => {
            const nodeId = root.id;
            append(`${nodeId}[${root.typename}]`, 4);
            if (root instanceof nunjucks.nodes.NodeList) {
                root.children.forEach((node, index) => {
                    const id = bindId(node);
                    append(`${nodeId} --"children[${index}]"--> ${id}`, 4);
                    walkAst(node);
                });
            }
            else if (root instanceof nunjucks.nodes.CallExtension) {
                const valueId = nodeIndex++;
                append(`${valueId}{{${root.extName}}}`, 4);
                append(`${nodeId} --extName--> ${valueId}`, 4);
                if (root.args) {
                    const id = bindId(root.args);
                    append(`${nodeId} --args--> ${id}`, 4);
                    walkAst(root.args);
                }
                if (root.contentArgs) {
                    root.contentArgs.forEach((n, index) => {
                        const id = bindId(n);
                        append(`${nodeId} --"contentArgs[${index}]"--> ${id}`, 4);
                        walkAst(n);
                    });
                }
            }
            else {
                root.iterFields((node, fieldName) => {
                    if (node instanceof nunjucks.nodes.Node) {
                        const id = bindId(node);
                        append(`${nodeId} --${fieldName}--> ${id}`, 4);
                        walkAst(node);
                    }
                    else {
                        const valueId = nodeIndex++;
                        append(`${valueId}{{"${String(node)
                            .replace(/(?:\r\n|\r|\n)/g, ' ')
                            .replace(/"/g, '&quot;')}"}}`, 4);
                        append(`${nodeId} --${fieldName}--> ${valueId}`, 4);
                    }
                });
            }
        };
        walkAst(ast);
        return output;
    }
}
exports.ASTPrinter = ASTPrinter;
//# sourceMappingURL=astPrinter.js.map